@@ -119,7 +119,8 @@ plevy <- function(q, m=0, s=1){
 		pts=as.integer(5),
 		max=as.integer(16),
 		err=integer(1),
-		res=double(len))
+		res=double(len),
+		DUP=F)
 	if(z$err==1)warning("Unable to allocate memory for integration")
 	if(z$err==2)warning("Division by zero in integration")
 	else if(z$err==3)warning("No convergence in integration")
@@ -167,7 +168,7 @@ pburr <- function(q, m, s, f){
 	if(any(m<=0))stop("m must be positive")
 	if(any(s<=0))stop("s must be positive")
 	if(any(f<=0))stop("f must be positive")
-	1-(1+(q/m)^s)^-f}
+	1-(1+(q/m)^s/f)^-f}
 
 dburr <- function(y, m, s, f){
 	if(any(y<0))stop("y must contain non-negative values")
@@ -175,7 +176,7 @@ dburr <- function(y, m, s, f){
 	if(any(s<=0))stop("s must be positive")
 	if(any(f<=0))stop("f must be positive")
 	y1 <- y/m
-	f*s*y1^(s-1)/(m^s*(1+y1^s)^(f+1))}
+	s*y1^(s-1)/(m^s*(1+y1^s/f)^(f+1))}
 
 # normed to make it a real distribution with y > 0
 pgextval <- function(q, s, m, f){
@@ -233,7 +234,8 @@ pginvgauss <- function(q, m, s, f){
 		pts=as.integer(5),
 		max=as.integer(16),
 		err=integer(1),
-		res=double(len))
+		res=double(len),
+		DUP=F)
 	if(z$err==1)warning("Unable to allocate memory for integration")
 	if(z$err==2)warning("Division by zero in integration")
 	else if(z$err==3)warning("No convergence in integration")
@@ -306,7 +308,8 @@ ppowexp <- function(q, m=0, s=1, f=1){
 		pts=as.integer(5),
 		max=as.integer(16),
 		err=integer(1),
-		res=double(len))
+		res=double(len),
+		DUP=F)
 	if(z$err==1)warning("Unable to allocate memory for integration")
 	if(z$err==2)warning("Division by zero in integration")
 	else if(z$err==3)warning("No convergence in integration")
@@ -332,12 +335,13 @@ pdoublepois <- function(q, m, s){
 		if(length(s)==1)s <- rep(s,length(q))
 		else stop("s and q must have the same length")}
 	.C("pdp",
-	as.integer(q),
-	as.integer(3*max(c(q,100))),
-	as.double(m),
-	as.double(s),
-	as.integer(length(q)),
-	res=double(length(q)))$res}
+		as.integer(q),
+		as.integer(3*max(c(q,100))),
+		as.double(m),
+		as.double(s),
+		as.integer(length(q)),
+		res=double(length(q)),
+		DUP=F)$res}
 
 ddoublepois <- function(y, m, s){
 	if(any(y<0))stop("y must contain non-negative values")
@@ -350,13 +354,14 @@ ddoublepois <- function(y, m, s){
 		if(length(s)==1)s <- rep(s,length(y))
 		else stop("s and y must have the same length")}
 	exp(.C("ddp",
-	as.integer(y),
-	as.integer(3*max(c(y,100))),
-	as.double(m),
-	as.double(s),
-	as.integer(length(y)),
-	as.double(rep(1,length(y))),
-	res=double(length(y)))$res)}
+		as.integer(y),
+		as.integer(3*max(c(y,100))),
+		as.double(m),
+		as.double(s),
+		as.integer(length(y)),
+		as.double(rep(1,length(y))),
+		res=double(length(y)),
+		DUP=F)$res)}
 
 pmultpois <- function(q, m, s){
 	if(any(q<0))stop("q must contain non-negative values")
@@ -369,12 +374,13 @@ pmultpois <- function(q, m, s){
 		if(length(s)==1)s <- rep(s,length(q))
 		else stop("s and q must have the same length")}
 	.C("pmp",
-	as.integer(q),
-	as.integer(3*max(c(q,100))),
-	as.double(m),
-	as.double(s),
-	as.integer(length(q)),
-	res=double(length(q)))$res}
+		as.integer(q),
+		as.integer(3*max(c(q,100))),
+		as.double(m),
+		as.double(s),
+		as.integer(length(q)),
+		res=double(length(q)),
+		DUP=F)$res}
 
 dmultpois <- function(y, m, s){
 	if(any(y<0))stop("y must contain non-negative values")
@@ -387,13 +393,14 @@ dmultpois <- function(y, m, s){
 		if(length(s)==1)s <- rep(s,length(y))
 		else stop("s and y must have the same length")}
 	exp(.C("dmp",
-	as.integer(y),
-	as.integer(3*max(c(y,100))),
-	as.double(m),
-	as.double(s),
-	as.integer(length(y)),
-	as.double(rep(1,length(y))),
-	res=double(length(y)))$res)}
+		as.integer(y),
+		as.integer(3*max(c(y,100))),
+		as.double(m),
+		as.double(s),
+		as.integer(length(y)),
+		as.double(rep(1,length(y))),
+		res=double(length(y)),
+		DUP=F)$res)}
 
 pgammacount <- function(q, m, s){
 	if(any(q<0))stop("q must contain non-negative values")
@@ -424,12 +431,13 @@ pdoublebinom <- function(q, n, m, s){
 		if(length(s)==1)s <- rep(s,length(q))
 		else stop("s and q must have the same length")}
 	.C("pdb",
-	as.integer(q),
-	as.integer(n),
-	as.double(m),
-	as.double(s),
-	as.integer(length(q)),
-	res=double(length(q)))$res}
+		as.integer(q),
+		as.integer(n),
+		as.double(m),
+		as.double(s),
+		as.integer(length(q)),
+		res=double(length(q)),
+		DUP=F)$res}
 
 ddoublebinom <- function(y, n, m, s){
 	if(any(y<0))stop("y must contain non-negative values")
@@ -447,13 +455,14 @@ ddoublebinom <- function(y, n, m, s){
 		if(length(s)==1)s <- rep(s,length(y))
 		else stop("s and y must have the same length")}
 	exp(.C("ddb",
-	as.integer(y),
-	as.integer(n),
-	as.double(m),
-	as.double(s),
-	as.integer(length(y)),
-	as.double(rep(1,length(y))),
-	res=double(length(y)))$res)}
+		as.integer(y),
+		as.integer(n),
+		as.double(m),
+		as.double(s),
+		as.integer(length(y)),
+		as.double(rep(1,length(y))),
+		res=double(length(y)),
+		DUP=F)$res)}
 
 pmultbinom <- function(q, n, m, s){
 	if(any(q<0))stop("q must contain non-negative values")
@@ -471,12 +480,13 @@ pmultbinom <- function(q, n, m, s){
 		if(length(s)==1)s <- rep(s,length(q))
 		else stop("s and q must have the same length")}
 	.C("pmb",
-	as.integer(q),
-	as.integer(n),
-	as.double(m),
-	as.double(s),
-	as.integer(length(q)),
-	res=double(length(q)))$res}
+		as.integer(q),
+		as.integer(n),
+		as.double(m),
+		as.double(s),
+		as.integer(length(q)),
+		res=double(length(q)),
+		DUP=F)$res}
 
 dmultbinom <- function(y, n, m, s){
 	if(any(y<0))stop("y must contain non-negative values")
@@ -494,13 +504,14 @@ dmultbinom <- function(y, n, m, s){
 		if(length(s)==1)s <- rep(s,length(y))
 		else stop("s and y must have the same length")}
 	exp(.C("dmb",
-	as.integer(y),
-	as.integer(n),
-	as.double(m),
-	as.double(s),
-	as.integer(length(y)),
-	as.double(rep(1,length(y))),
-	res=double(length(y)))$res)}
+		as.integer(y),
+		as.integer(n),
+		as.double(m),
+		as.double(s),
+		as.integer(length(y)),
+		as.double(rep(1,length(y))),
+		res=double(length(y)),
+		DUP=F)$res)}
 
 pbetabinom <- function(q, n, m, s){
 	if(any(q<0))stop("q must contain non-negative values")
